@@ -7,6 +7,7 @@ import logging
 import click
 
 import koapy
+
 from koapy.utils.logging import set_verbosity
 
 CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
@@ -49,7 +50,7 @@ def serve(port, verbose, no_verbose, args):
 @click.option('-v', '--verbose', count=True, help='Verbosity.')
 def login(port, verbose):
     set_verbosity(verbose)
-    from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
+    from koapy import KiwoomOpenApiContext
     with KiwoomOpenApiContext(port=port, client_check_timeout=client_check_timeout) as context:
         state = context.GetConnectState()
         if state == 0:
@@ -72,7 +73,7 @@ def config():
 @click.option('-v', '--verbose', count=True, help='Verbosity.')
 def autologin(port, verbose):
     set_verbosity(verbose)
-    from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
+    from koapy import KiwoomOpenApiContext
     with KiwoomOpenApiContext(port=port, client_check_timeout=client_check_timeout) as context:
         context.EnsureConnected()
         context.ShowAccountWindow()
@@ -139,7 +140,7 @@ def stockcode(markets, port):
     if (markets) == (tuple()):
         fail_with_usage()
 
-    from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
+    from koapy import KiwoomOpenApiContext
 
     if 'all' in markets:
         markets = market_codes
@@ -161,7 +162,7 @@ def stockcode(markets, port):
 @click.option('-c', '--code', 'codes', metavar='CODE', multiple=True, help='Stock code to get. Can set multiple times.')
 @click.option('-p', '--port', metavar='PORT', help='Port number of grpc server (optional).')
 def stockname(codes, port):
-    from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
+    from koapy import KiwoomOpenApiContext
 
     with KiwoomOpenApiContext(port=port, client_check_timeout=client_check_timeout) as context:
         context.EnsureConnected()
@@ -199,7 +200,7 @@ def stockname(codes, port):
 @click.option('-f', '--format', metavar='FORMAT', type=click.Choice(['md', 'xlsx', 'json'], case_sensitive=False))
 @click.option('-p', '--port', metavar='PORT', help='Port number of grpc server (optional).')
 @click.option('-v', '--verbose', count=True, help='Verbosity.')
-def stockinfo(codes, markets, input, output, format, port, verbose):
+def stockinfo(codes, markets, input, output, format, port, verbose): # pylint: disable=redefined-builtin
     """
     \b
     Possible market codes are:
@@ -238,7 +239,7 @@ def stockinfo(codes, markets, input, output, format, port, verbose):
         if os.path.isfile(input):
             if input.endswith('.xlsx'):
                 import pandas as pd
-                df = pd.read_excel(input, dtype='object')
+                df = pd.read_excel(input, dtype=str)
                 code_column = '종목코드'
                 if code_column in df:
                     codes = df[code_column]
@@ -274,7 +275,7 @@ def stockinfo(codes, markets, input, output, format, port, verbose):
 
     import pandas as pd
 
-    from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
+    from koapy import KiwoomOpenApiContext
 
     with KiwoomOpenApiContext(port=port, client_check_timeout=client_check_timeout, verbosity=verbose) as context:
         context.EnsureConnected()
@@ -324,7 +325,7 @@ def stockinfo(codes, markets, input, output, format, port, verbose):
 @click.option('-e', '--end-date', metavar='YYYY-MM-DD', type=click.DateTime(formats=['%Y-%m-%d', '%Y%m%d']), help='Stops if reached, not included (optional).')
 @click.option('-p', '--port', metavar='PORT', help='Port number of grpc server (optional).')
 @click.option('-v', '--verbose', count=True, help='Verbosity.')
-def daily(codes, input, output, start_date, end_date, port, verbose):
+def daily(codes, input, output, start_date, end_date, port, verbose): # pylint: disable=redefined-builtin
     if (codes, input, output, start_date, end_date) == (tuple(), None, None, None, None):
         fail_with_usage()
 
@@ -341,7 +342,7 @@ def daily(codes, input, output, start_date, end_date, port, verbose):
         if os.path.isfile(input):
             if input.endswith('.xlsx'):
                 import pandas as pd
-                df = pd.read_excel(input, dtype='object')
+                df = pd.read_excel(input, dtype=str)
                 code_column = '종목코드'
                 if code_column in df:
                     codes = df[code_column]
@@ -390,7 +391,7 @@ def daily(codes, input, output, start_date, end_date, port, verbose):
     import datetime
     import pandas as pd
 
-    from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
+    from koapy import KiwoomOpenApiContext
 
     with KiwoomOpenApiContext(port=port, client_check_timeout=client_check_timeout, verbosity=verbose) as context:
         context.EnsureConnected()
@@ -398,15 +399,21 @@ def daily(codes, input, output, start_date, end_date, port, verbose):
         for i, code in enumerate(codes):
             logging.info('Starting to get stock data for code: %s (%d/%d)', code, i+1, codes_len)
             filepath = filepath_for_code(code)
+            appended = False
             if os.path.exists(filepath):
-                df = pd.read_excel(filepath, dtype='object')
-                last_date = df.loc[0, '일자']
-                last_date = datetime.datetime.strptime(last_date, '%Y%m%d')
-                logging.info('Found existing file %s, prepending from %s until %s', os.path.basename(filepath), start_date, last_date)
-                df = pd.concat([context.GetDailyStockDataAsDataFrame(code, start_date, last_date), df], sort=False)
-            else:
+                df = pd.read_excel(filepath, dtype=str)
+                if df.shape[0] > 0:
+                    last_date = df.loc[0, '일자']
+                    last_date = datetime.datetime.strptime(last_date, '%Y%m%d')
+                    logging.info('Found existing file %s, prepending from %s until %s', os.path.basename(filepath), start_date, last_date)
+                    df = pd.concat([context.GetDailyStockDataAsDataFrame(code, start_date, last_date), df], sort=False)
+                    tempfilepath = filepath + '.tmp'
+                    df.to_excel(tempfilepath, index=False)
+                    os.replace(tempfilepath, filepath)
+                    appended = True
+            if not appended:
                 df = context.GetDailyStockDataAsDataFrame(code, start_date, end_date)
-            df.to_excel(filepath, index=False)
+                df.to_excel(filepath, index=False)
             logging.info('Saved stock data for code %s to %s', code, filepath)
 
 minute_intervals = [
@@ -429,7 +436,7 @@ minute_intervals = [
 @click.option('-e', '--end-date', metavar="YYYY-MM-DD['T'hh:mm:ss]", type=click.DateTime(formats=['%Y-%m-%d', '%Y%m%d', '%Y-%m-%dT%H:%M:%S', '%Y%m%d%H%M%S']), help='Stops if reached, not included (optional).')
 @click.option('-p', '--port', metavar='PORT', help='Port number of grpc server (optional).')
 @click.option('-v', '--verbose', count=True, help='Verbosity.')
-def minute(codes, interval, input, output, start_date, end_date, port, verbose):
+def minute(codes, interval, input, output, start_date, end_date, port, verbose): # pylint: disable=redefined-builtin
     if (codes, interval, input, output, start_date, end_date) == (tuple(), None, None, None, None, None):
         fail_with_usage()
 
@@ -449,7 +456,7 @@ def minute(codes, interval, input, output, start_date, end_date, port, verbose):
         if os.path.isfile(input):
             if input.endswith('.xlsx'):
                 import pandas as pd
-                df = pd.read_excel(input, dtype='object')
+                df = pd.read_excel(input, dtype=str)
                 code_column = '종목코드'
                 if code_column in df:
                     codes = df[code_column]
@@ -498,7 +505,7 @@ def minute(codes, interval, input, output, start_date, end_date, port, verbose):
     import datetime
     import pandas as pd
 
-    from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
+    from koapy import KiwoomOpenApiContext
 
     with KiwoomOpenApiContext(port=port, client_check_timeout=client_check_timeout, verbosity=verbose) as context:
         context.EnsureConnected()
@@ -506,15 +513,21 @@ def minute(codes, interval, input, output, start_date, end_date, port, verbose):
         for i, code in enumerate(codes):
             logging.info('Starting to get stock data for code: %s (%d/%d)', code, i+1, codes_len)
             filepath = filepath_for_code(code)
+            appended = False
             if os.path.exists(filepath):
-                df = pd.read_excel(filepath, dtype='object')
-                last_date = df.loc[0, '체결시간']
-                last_date = datetime.datetime.strptime(last_date, '%Y%m%d%H%M%S')
-                logging.info('Found existing file %s, prepending from %s until %s', os.path.basename(filepath), start_date, last_date)
-                df = pd.concat([context.GetMinuteStockDataAsDataFrame(code, interval, start_date, last_date), df], sort=False)
-            else:
+                df = pd.read_excel(filepath, dtype=str)
+                if df.shape[0] > 0:
+                    last_date = df.loc[0, '체결시간']
+                    last_date = datetime.datetime.strptime(last_date, '%Y%m%d%H%M%S')
+                    logging.info('Found existing file %s, prepending from %s until %s', os.path.basename(filepath), start_date, last_date)
+                    df = pd.concat([context.GetMinuteStockDataAsDataFrame(code, interval, start_date, last_date), df], sort=False)
+                    tempfilepath = filepath + '.tmp'
+                    df.to_excel(tempfilepath, index=False)
+                    os.replace(tempfilepath, filepath)
+                    appended = True
+            if not appended:
                 df = context.GetMinuteStockDataAsDataFrame(code, interval, start_date, end_date)
-            df.to_excel(filepath, index=False)
+                df.to_excel(filepath, index=False)
             logging.info('Saved stock data for code %s to %s', code, filepath)
 
 @get.command(context_settings=CONTEXT_SETTINGS, short_help='Get TR info.')
@@ -594,15 +607,27 @@ def realinfo(realtypes):
                 click.echo('  [%s] = %s' % (fid, name))
 
 @get.command(context_settings=CONTEXT_SETTINGS, short_help='Get market holidays.')
-@click.option('-o', '--output', metavar='FILENAME', type=click.Path(), help='Output filename (optional).')
+@click.option('-o', '--output', metavar='FILENAME', type=click.Path(), help='Output filename. (optional)')
+@click.option('-O', '--offline', is_flag=True, help='Do not use krx marketdata api. (default: false)')
+@click.option('-u', '--update', is_flag=True, help='Update local cache, download from krx marketdata api.')
+@click.option('-U', '--no-update', is_flag=True, help='Disable update.')
 @click.option('-v', '--verbose', count=True, help='Verbosity.')
-def holidays(output, verbose):
+def holidays(output, offline, update, no_update, verbose):
     set_verbosity(verbose)
 
     if output is None:
         import pandas as pd
-        from koapy.utils.krx.holiday import get_holidays_as_dict
-        response = get_holidays_as_dict()
+        if not offline:
+            from koapy.utils.krx.marketdata.holiday import download_holidays_as_dict
+            response = download_holidays_as_dict()
+            if not no_update:
+                from koapy.utils.krx.holiday.KrxBusinessDay import dump_holidays
+                dump_holidays(response)
+        else:
+            from koapy.utils.krx.holiday import get_holidays_as_dict
+            response = get_holidays_as_dict()
+            if not no_update or update:
+                logging.warning('Cannot update while offline.')
         lang = locale.getdefaultlocale()[0]
         if lang == 'ko_KR':
             day_key = 'kr_dy_tp'
@@ -619,10 +644,15 @@ def holidays(output, verbose):
         df = pd.DataFrame.from_records(data, columns=columns)
         click.echo(df.to_markdown())
     else:
-        from koapy.utils.krx.holiday import download_holidays_as_excel
-        if output and not output.endswith('.xls'):
-            output += '.xls'
-        download_holidays_as_excel(output)
+        if not offline:
+            from koapy.utils.krx.marketdata.holiday import download_holidays_as_excel
+            if output and not output.endswith('.xls'):
+                output += '.xls'
+            download_holidays_as_excel(output)
+            if update:
+                logging.warning('Cannot update on file output.')
+        else:
+            fail_with_usage('Saving to file should come with offline option disabled.')
 
 @get.command(context_settings=CONTEXT_SETTINGS, short_help='Get user information.')
 @click.option('-p', '--port', metavar='PORT', help='Port number of grpc server (optional).')
@@ -631,7 +661,7 @@ def userinfo(port, verbose):
     set_verbosity(verbose)
 
     import pandas as pd
-    from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
+    from koapy import KiwoomOpenApiContext
 
     with KiwoomOpenApiContext(port=port, client_check_timeout=client_check_timeout, verbosity=verbose) as context:
         context.EnsureConnected()
@@ -668,7 +698,7 @@ def deposit(account, port, verbose):
     if account is None:
         logging.info('Account not given. Using first account available.')
 
-    from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
+    from koapy import KiwoomOpenApiContext
 
     with KiwoomOpenApiContext(port=port, client_check_timeout=client_check_timeout, verbosity=verbose) as context:
         context.EnsureConnected()
@@ -702,7 +732,7 @@ def evaluation(account, include_delisted, exclude_delisted, for_each, as_summary
     elif for_each:
         lookup_type = '2'
 
-    from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
+    from koapy import KiwoomOpenApiContext
 
     with KiwoomOpenApiContext(port=port, client_check_timeout=client_check_timeout, verbosity=verbose) as context:
         context.EnsureConnected()
@@ -745,7 +775,7 @@ def orders(account, date, reverse, executed_only, not_executed_only, stock_only,
     if account is None:
         logging.info('Account not given. Using first account available.')
 
-    from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
+    from koapy import KiwoomOpenApiContext
 
     sort_type = '1'
     if reverse:
@@ -778,7 +808,7 @@ def orders(account, date, reverse, executed_only, not_executed_only, stock_only,
 @click.option('-v', '--verbose', count=True, help='Verbosity.')
 def modulepath(port, verbose):
     set_verbosity(verbose)
-    from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
+    from koapy import KiwoomOpenApiContext
     with KiwoomOpenApiContext(port=port, client_check_timeout=client_check_timeout, verbosity=verbose) as context:
         click.echo(context.GetAPIModulePath())
 
@@ -817,7 +847,7 @@ def watch(codes, input, fids, realtype, output, format, port, verbose):
         if os.path.isfile(input):
             if input.endswith('.xlsx'):
                 import pandas as pd
-                df = pd.read_excel(input, dtype='object')
+                df = pd.read_excel(input, dtype=str)
                 code_column = '종목코드'
                 if code_column in df:
                     codes = df[code_column]
@@ -847,7 +877,7 @@ def watch(codes, input, fids, realtype, output, format, port, verbose):
     import datetime
     import pandas as pd
 
-    from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
+    from koapy import KiwoomOpenApiContext
     from koapy.openapi.RealType import RealType
 
     def parse_message(message):
@@ -918,13 +948,16 @@ def order(request_name, screen_no, account_no, order_type, code, quantity, price
     """
     \b
     [주문유형]
-      1:신규매수, 2:신규매도 3:매수취소, 4:매도취소, 5:매수정정, 6:매도정정
+      1 : 신규매수
+      2 : 신규매도
+      3 : 매수취소
+      4 : 매도취소
+      5 : 매수정정
+      6 : 매도정정
 
     \b
     [거래구분]
       모의투자에서는 지정가 주문과 시장가 주문만 가능합니다.
-
-    \b
       00 : 지정가
       03 : 시장가
       05 : 조건부지정가
@@ -946,7 +979,7 @@ def order(request_name, screen_no, account_no, order_type, code, quantity, price
 
     set_verbosity(verbose)
 
-    from koapy.context.KiwoomOpenApiContext import KiwoomOpenApiContext
+    from koapy import KiwoomOpenApiContext
     from google.protobuf.json_format import MessageToDict
 
     if format == 'json':
