@@ -5,7 +5,7 @@ import logging
 import numpy as np
 
 from koapy.openapi.KiwoomOpenApiError import KiwoomOpenApiError
-from koapy.utils.rate_limiting.RateLimiter import SimpleRateLimiter
+from koapy.utils.rate_limiting.RateLimiter import KiwoomRateLimiter
 
 class KiwoomOpenApiControlCommonWrapper:
 
@@ -42,7 +42,7 @@ class KiwoomOpenApiControlCommonWrapper:
         states = states.split('|') if states else []
         return states
 
-    def GetCommonCodeList(self):
+    def GetKospiCodeList(self, include_preferred_stocks=False, include_reits=True):
         """
         [시장구분값]
           0 : 장내
@@ -60,6 +60,10 @@ class KiwoomOpenApiControlCommonWrapper:
         # 기본 코드 리스트
         codes = self.GetCodeListByMarketAsList('0')
 
+        # 코드 마지막 자리가 0 이 아니면 우선주일 가능성이 있다고 보고 제외
+        if not include_preferred_stocks:
+            codes = [code for code in codes if code.endswith('0')]
+
         # 장내 시장에서 ETN 이 섞여 있는데 시장구분값으로 뺄 수가 없어서 이름을 보고 대충 제외
         names = [self.GetMasterCodeName(code) for code in codes]
         etn_suffixes = ['ETN', 'ETN(H)', 'ETN B', 'ETN(H) B']
@@ -69,18 +73,14 @@ class KiwoomOpenApiControlCommonWrapper:
         # 코드값 기준 제외 준비
         codes = set(codes)
 
-        # 우선주 구분은 기본정보에서 유통주식 수량이 확인되지 않는 주식들
-        # 그 중에 ~3호 등 배 관련 종목은 우선주가 아니고 PER 같은 값이 있음
-        # 종목코드의 마지막 자리가 숫자가 아닌 K,L 등 인 것으로 구분할 수도 있는데 모두 그런건 아님
-        # 당장은 미리  구분해놓은 값들을 참고해 제외하는 식으로
-        preferred_stock_codes = []
-        codes = codes - set(preferred_stock_codes)
-
         # 나머지는 혹시나 겹치는 애들이 나올 수 있는 시장에서 코드기준 제외
         codes = codes - set(self.GetCodeListByMarketAsList('10'))
         codes = codes - set(self.GetCodeListByMarketAsList('8'))
         codes = codes - set(self.GetCodeListByMarketAsList('4'))
-        codes = codes - set(self.GetCodeListByMarketAsList('6'))
+
+        # 리츠는 기본적으로 코스피에 포함됨
+        if not include_reits:
+            codes = codes - set(self.GetCodeListByMarketAsList('6'))
 
         # 정렬된 리스트 형태로 제공
         codes = sorted(list(codes))
@@ -92,6 +92,9 @@ class KiwoomOpenApiControlCommonWrapper:
         codes = sorted(codes)
         return codes
 
+    def GetCommonCodeList(self):
+        return self.GetKospiCodeList(include_preferred_stocks=True, include_reits=False)
+
     def IsSuspended(self, code):
         return '거래정지' in self.GetMasterStockStateAsList(code)
 
@@ -128,10 +131,17 @@ class KiwoomOpenApiControlWrapper(KiwoomOpenApiControlCommonWrapper):
                 self.OnEventConnect.disconnect(OnEventConnect)
             self.OnEventConnect.connect(OnEventConnect)
             errcode = KiwoomOpenApiError.try_or_raise(self.CommConnect())
-            errcode = q.get()
+            errcode = KiwoomOpenApiError.try_or_raise(q.get())
         return errcode
 
-    @SimpleRateLimiter(period=4, calls=1) # 그냥 1초당 5회로하면 장기적으로 결국 막히기 때문에 4초당 1회로 제한 (3초당 1회부턴 제한걸림)
+    # 그냥 1초당 5회로하면 장기적으로 결국 막히기 때문에 원래 4초당 1회로 제한했었음 (3초당 1회부턴 제한걸림)
+    # @SimpleRateLimiter(period=4, calls=1)
+
+    # 1시간에 1000회로 제한한다는 추측이 있는데 일리 있어 보임 (http://blog.quantylab.com/htsapi.html)
+    # 1초당 1회로 계산했을때 1시간이면 3600 회, 주기를 1초씩 늘려보면
+    # 2초당 1회 => 1800 > 1000, 3초당 1회 => 1200 > 1000, 4초당 1회 => 900 < 1000
+
+    @KiwoomRateLimiter()
     def RateLimitedCommRqData(self, rqname, trcode, prevnext, scrnno, inputs=None):
         """
         [OpenAPI 게시판]
